package com.medias.annuaire;

import com.medias.ConversionException;
import com.medias.utils.Entity;
import com.medias.xml.XMLBuilder;
import com.medias.xml.metadata.Contact;
import com.medias.xml.metadata.Metadata;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.jdom.Element;
import org.jdom.JDOMException;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @author combaz
 *
 * Created on 22 févr. 2005
 */
public class Annuaire implements Entity
{
	/**
     * 
     */
    private static final long serialVersionUID = 1L;

    /**
	 * L'attribut que l'on prend pour constituer le login
	 */
	public static String LOGIN_FIELD = "mail";
	
	public static String KWD_FIELD = "mdp";
	
	private String xmlFileName;
	
	private ArrayList<Object> contacts;
	
	public Annuaire(String file) {
		this.xmlFileName = file;
		this.contacts = new ArrayList<Object>();
		try {
			this.update();
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	/**
	 * @return Returns the contacts.
	 */
	public Object[] getContacts() {
		if(this.contacts.isEmpty()) {
			Contact pers = new Contact();
			this.contacts.add(pers);
		}
		return this.contacts.toArray();
	}
	/**
	 * @param contacts The contacts to set.
	 */
	public void setContacts(Contact[] contacts) {
		this.contacts = new ArrayList<Object>();
		for(int i=0; i<contacts.length; i++)
			this.contacts.add(contacts[i]);
	}
	/**
	 * @return Returns the xmlFileName.
	 */
	public String getXmlFileName() {
		return xmlFileName;
	}
	/**
	 * @param xmlFileName The xmlFileName to set.
	 */
	public void setXmlFileName(String xmlFileName) {
		this.xmlFileName = xmlFileName;
	}
	
	public void fromXmlElement(Element el) throws ConversionException {
		List<?> p = el.getChildren("personne");
		Iterator<?> it = p.iterator();
		
		if((p==null)) {
			throw new ConversionException(ConversionException.ERROR_fromXml + "Annuaire !");
		}

		while(it.hasNext()){
			Element child = (Element)it.next();
			Personne pers = new Personne();
			pers.fromXmlElement(child);
			this.contacts.add(pers);
		}
	}
	
	public Element toXmlElement(String title) {
		ArrayList<Element> pl = new ArrayList<Element>();
		for(int i = 0; i<this.contacts.size(); i++) {
			Personne pers = (Personne)this.contacts.get(i);
			Element p = pers.toXmlElement("personne");
			pl.add(p);
		}
		Element result = new Element(title);
		result.addContent(pl);
		
		return result;
	}
	
	public ArrayList<Element> getPersonnes(String login) throws IOException, JDOMException {
		ArrayList<Element> result = new ArrayList<Element>();
		XMLBuilder builder = new XMLBuilder();
		builder.readXml(this.xmlFileName);
		List<?> personnes = builder.getRoot().getChildren("personne");
		Iterator<?> it = personnes.iterator();
		
		while(it.hasNext()) {
			Element pers = (Element) it.next();
			Element key = pers.getChild(Annuaire.LOGIN_FIELD);
			if(key.getText().equals(login))
				result.add(pers);
		}
		
		return result;
	}
	
	public Personne getPersonne(String login, String pwd) throws IOException, JDOMException, ConversionException {
		ArrayList<Element> selection = this.getPersonnes(login);
		Iterator<Element> it = selection.iterator();
		
		while(it.hasNext()) {
			Element pers = it.next();
			if(pers.getChild(Annuaire.KWD_FIELD).getText().equals(pwd)) {
				Personne p = new Personne();
				p.fromXmlElement(pers);
				return p;
			}
		}
		
		return null;
	}
	
//	private boolean jeuHasRole (String jeu, String role) {
//		/* Recherche d'un contact ayant le role dans le jeu */
//		Iterator it = this.contacts.iterator();
//		while(it.hasNext()) {
//			Contact pers = (Contact) it.next();
//			if(pers.getRole().equalsIgnoreCase(role)) {
//				return true;
//			}
//		}
//		
//		return false;
//	}
	
	/**
	 * A partir d'un contact récupéré dans une métadonnée, va ajouter la personne correspondante à l'annuaire si elle
	 * n'existe pas, ou bien, dans le cas contraire, remettre simplement à jour la liste de jeux qui lui est associée.
	 * Pour un jeu donné, il ne peut y avoir qu'une seule personne par rôle.
	 * @param c Le contact à ajouter
	 * @param jeu Le jeu associé à ce contact.
	 * @throws IOException
	 */
	public boolean addPersonne(Contact c, String jeu, ActionMessages messages) throws IOException{
		/* On vérifie si une autre personne détient le rôle de la personne à ajouter au niveau du jeu. Si tel est le 
		 * cas, la personne n'est ni ajouté à l'annuaire, ni au jeu */
		
//		String role = c.getRole();
//		if (!jeuHasRole (jeu, role)) {
			Personne p = new Personne();
			p.fromContact(c);
			ArrayList<String> l = new ArrayList<String>();
			l.add(jeu);
			p.setJeux(l);
			boolean res = this.addPersonne(p,messages);
			c.setMdp(p.getMdp());
			return res;
//		}
	}
	
	/**
	 * Ajoute une personne à l'annuaire si elle n'existe pas encore, ou compléter la liste de jeux qui lui est associée.
	 * @param p la personne à ajouter
	 * @return true si la personne a bien été ajoutée, false si la personne existait déjà.
	 */
	private boolean addPersonne(Personne p, ActionMessages messages) throws IOException {
		boolean result = false;
		
		if(!this.contains(p)){
			this.contacts.add(p);
			result = true;
		}
		else {
			Iterator<Object> it = this.contacts.iterator();
			//on trouve l'enregistrement dans l'annuaire correspondant à la personne à mettre à jour
			while(it.hasNext()){
				Personne pers = (Personne)it.next();
				if(pers.equals(p)) {
					// Si la personne existe déjà mais que le mot de passe saisi est différent, on l'indique par un message d'info
					if (!pers.getMdp().toLowerCase().equals(p.getMdp().trim().toLowerCase())) {
						messages.add("warning", new ActionMessage("data.upload.newSet.pwdexist",pers.getMail(),pers.getMdp()));
						p.setMdp(pers.getMdp());
					}
					Iterator<String> pit = p.getJeux().iterator();
					ArrayList<String> l = pers.getJeux();
					//on parcourt les jeux à rajouter et on ajoute ceux qui manquent
					while(pit.hasNext()) {
						String jeu = (String) pit.next();
						if(!l.contains(jeu)) {
							l.add(jeu);
						}
					}
					pers.setJeux(l);
				}
					
			}
		}
		return result;
	}
	
	public void createXmlFile() throws IOException {
		Element root = this.toXmlElement("annuaire");
		XMLBuilder builder = new XMLBuilder(root);
		builder.writeXml(this.getXmlFileName());
	}

	/**
	 * Remet à jour le fichier com.medias.xml d'annuaire à partir des contacts associés à un jeu de données.
	 * @param metadata	Les métadonnées associées au jeu, dans lesquelles on va trouver les contacts en question
	 * @throws IOException
	 */
	public int update(Metadata metadata, ActionMessages messages) throws IOException {
		//le nom du jeu, à associer aux contacts dans l'annuaire,
		String jeu = metadata.getPres().getNom();
		Iterator<Contact> meta_it = metadata.getPres().getContactsList().iterator();
		//pour chacun des contacts trouvés, on complète l'annuaire,
		int nbPersonnesAPrevenir=0;
		while(meta_it.hasNext()) {
			Contact pers = meta_it.next();
			if((pers.getMail() != null) && (pers.getMail().length() != 0)) {
				this.addPersonne(pers,jeu,messages);
				nbPersonnesAPrevenir++;
			}
		}
		
		//puis, on vérifie la cohérence (en cas de suppression de contacts associés au jeu)
		Iterator<Object> annu_it = this.contacts.iterator();
		while(annu_it.hasNext()) {
			Personne pers = (Personne) annu_it.next();
			Iterator<String> jeu_it = pers.getJeux().iterator();
			boolean remove = false;
			while((!remove)&&(jeu_it.hasNext())) {
				String jeu_pers = (String) jeu_it.next();
				if(jeu_pers.equals(jeu)) {
					remove = true;
					meta_it = metadata.getPres().getContactsList().iterator();
					while(meta_it.hasNext()) {
						Contact contact = meta_it.next();
						Personne p = new Personne();
						p.fromContact(contact);
						if(pers.equals(p))
							remove = false;
					}
					if(remove==true) {
						pers.getJeux().remove(jeu);
					}
				}
			}
		}
		this.createXmlFile();
		return nbPersonnesAPrevenir;
	}
	
	public void update() throws IOException, JDOMException, ConversionException {
		XMLBuilder builder = new XMLBuilder();
		builder.readXml(this.xmlFileName);
		Element root = builder.getRoot();
		this.fromXmlElement(root);
	}
	
	public boolean contains(Personne p) {
		Iterator<Object> it = this.contacts.iterator();
		while(it.hasNext()){
			Personne pers = (Personne) it.next();
			if(pers.equals(p))
				return true;
		}
		return false;
	}
	
	public void deleteSet(String setName) throws IOException {
		Iterator<Object> pers_it = this.contacts.iterator();
		while(pers_it.hasNext()) {
			Personne pers = (Personne) pers_it.next();
			pers.removeSet(setName);
		}
		this.createXmlFile();
	}
}
