package com.medias.xml.metadata;

import com.medias.Constantes;
import com.medias.ConversionException;
import com.medias.utils.Entity;
import org.jdom.Element;

import java.util.Comparator;

/**
 * @author combaz
 *
 * Created on 10 févr. 2005
 */
public class Capteur implements Entity
{
	/**
     * 
     */
    private static final long serialVersionUID = 1L;
    private String type;
    private String typeOth;
	private String fabriquant;
	private String modele;
	private String num;
	private String etalon;
	// Utilisé dans la partie dépôt de données
	private String nomPlateforme;
	// Utilisé dans la partie accès aux données
	private String desc;
	private String idCapteur;
	
	public Capteur() {
		super();
		this.type = "";
		this.typeOth = "";
		this.fabriquant = "";
		this.modele = "";
		this.num = "";
		this.etalon = null;
		this.desc = null;
		this.idCapteur = null;
		this.nomPlateforme = "";
	}
	
    public String getIdCapteur() {
        return idCapteur;
    }
    
    public void setIdCapteur(String idCapteur) {
        this.idCapteur = idCapteur;
    }
    
    public String getDesc() {
        return desc;
    }
    
    public void setDesc(String desc) {
        this.desc = desc;
    }
    
	/**
	 * @return Returns the etalon.
	 */
	public String getEtalon() {
		return etalon;
	}
	/**
	 * @param etalon The etalon to set.
	 */
	public void setEtalon(String etalon) {
		this.etalon = etalon;
	}
	/**
	 * @return Returns the fabriquant.
	 */
	public String getFabriquant() {
		return fabriquant;
	}
	/**
	 * @param fabriquant The fabriquant to set.
	 */
	public void setFabriquant(String fabriquant) {
		this.fabriquant = fabriquant;
	}
	/**
	 * @return Returns the modele.
	 */
	public String getModele() {
		return modele;
	}
	/**
	 * @param modele The modele to set.
	 */
	public void setModele(String modele) {
		this.modele = modele;
	}
	/**
	 * @return Returns the num.
	 */
	public String getNum() {
		return num;
	}
	/**
	 * @param num The num to set.
	 */
	public void setNum(String num) {
		this.num = num;
	}
	/**
	 * @return Returns the type.
	 */
	public String getType() {
		return this.type;
	}
	/**
	 * @param type The type to set.
	 */
	public void setType(String type) {
		this.type = type;
	}
	
    public String getNomPlateforme() {
        return nomPlateforme;
    }
    
    public void setNomPlateforme(String nomPlateforme) {
        this.nomPlateforme = nomPlateforme;
    }
    
	public Element toXmlElement(String title) {
		Element type = new Element("type");
		type.addContent(this.type);
		Element fab = new Element("fabriquant");
		fab.addContent(this.fabriquant);
		Element modele = new Element("modele");
		modele.addContent(this.modele);
		Element nums = new Element("num_serie");
		nums.addContent(this.num);
		Element etalon = new Element("etalonnage");
		etalon.addContent(this.etalon);
		Element nomPlat = new Element("plateforme");
		if (this.nomPlateforme != null)
		    nomPlat.addContent(this.nomPlateforme);
		else
		    nomPlat.addContent("");
		Element capteur = new Element(title);
		capteur.addContent(type);
		capteur.addContent(fab);
		capteur.addContent(modele);
		capteur.addContent(nums);
		capteur.addContent(etalon);
		capteur.addContent(nomPlat);
		return capteur;
	}
	
	public void fromXmlElement(Element el) throws ConversionException {
		Element t = el.getChild("type");
		Element f = el.getChild("fabriquant");
		Element m = el.getChild("modele");
		Element n = el.getChild("num_serie");
		Element e = el.getChild("etalonnage");
		Element p = el.getChild("plateforme");
		
		if((t==null)||(f==null)||(m==null)||(n==null)||(e==null)) {
			throw new ConversionException(ConversionException.ERROR_fromXml + "Capteur !");
		}
		else {
			boolean isTypeDefault = false;
			for (String typeCapt: Constantes.listeTypeCapteurs) {
				if (t.getText().equals(typeCapt)) {
					isTypeDefault = true;
					break;
				}
			}
			if (isTypeDefault)
				this.setType(t.getText());
			else {
				this.setType("-");
				this.setTypeOth(t.getText());
			}
			this.setFabriquant(f.getText());
			this.setModele(m.getText());
			this.setNum(n.getText());
			this.setEtalon(e.getText());
			if (p==null) {
			    p = new Element("plateforme");
			    p.addContent("");
			    this.setNomPlateforme(p.getText());
			} else {
			    this.setNomPlateforme(p.getText());
			}
		}
	}
	
	public void formToModel() {
		etalon = ( etalon == null ? "" : etalon.trim() );
		fabriquant = ( fabriquant == null ? "" : fabriquant.trim() );
		modele = ( modele == null ? "" : modele.trim() );
		num = ( num == null ? "" : num.trim() );
//		type = ( type == null ? "" : type.trim() );
		if (type == null || type.equals("-")) {
			if (typeOth == null) {
				type = "";
			} else {
				type = typeOth.trim();
			}
		} else {
			type = type.trim();
		}
	}
	
    public class ComparatorNom implements Comparator<Capteur>{
        public int compare(Capteur c1, Capteur c2){
        	if (c1.getType().equals(c2.getType()))
        		return c1.getModele().compareTo(c2.getModele());
        	else
        		return c1.getType().compareTo(c2.getType());
        }
    }

	public String getTypeOth() {
		return typeOth;
	}

	public void setTypeOth(String typeOth) {
		this.typeOth = typeOth;
	}
}
